#
#  finance.yahoo.com stock service
#  yahoo provides a nice 'csv' format, easy to parse
#  decimal values returned by yahoo

proc finance.yahoo.com {} {
    global ticker_symbols data
    set i 0
    set symbol_list ""
    while {[info exists ticker_symbols([incr i])]} {
	lappend symbol_list $ticker_symbols($i)
    }
    if {[llength $symbol_list] == 0} {
        return 0
    }
    set symvar [cgi_cgi_set s $symbol_list]
    set timeout [expr {3 * 60000}]
    set id [http::geturl \
        http://finance.yahoo.com/d/quotes.csv?f=sl1d1t1c1ohgv&e=.csv&$symvar \
	-timeout $timeout]
    if {"[http::wait $id]" != "ok"} {
        catch {unset $id}
	return 0
    }
    set html [http::data $id]
    catch {unset $id}
    foreach line [split $html \n] {
	# further split on comma to separate quote values
	set datavals [split $line ,]
	if {[llength $datavals] != 9} continue
	foreach {sym last date time change bid ask open vol} $datavals {
	    break
	}
	# remove double quotes around symbols
	regsub -all \" $sym {} sym
	# make sure symbol and quote values are valid
	if {[string length $sym] == 0} continue
	if {[scan $last   %f v] != 1}  continue
	if {[scan $change %f v] != 1}  continue

	set newdata($sym,last) $last
        if {$change > 0} {
	    set newdata($sym,sign) +
	} elseif {$change < 0} {
	    set newdata($sym,sign) -
	} else {
	    set newdata($sym,sign) =
	}
	set newdata($sym,delta) $change
    }
    catch {unset data}
    global data
    array set data [array get newdata]
    return 1
}


proc finance.yahoo.com_lookup {str type} {
    if {[string length $str] == 0} {
        return ""
    }
    switch $type {
	any     {set type ""}
        stock   {set type S}
	option  {set type O}
	bond    {set type ""}
	mutual  {set type M}
	index   {set type I}
	default {set type ""}
    }
    set symco_list ""
    set symvar [cgi_cgi_set s $str]
    set timeout [expr {3 * 60000}]
    set id [http::geturl http://finance.yahoo.com/l?m=US&t=$type&$symvar -timeout $timeout]
    if {"[http::wait $id]" != "ok"} {
        catch {unset $id}
	return ""
    }
    set html [http::data $id]
    catch {unset $id}
    
    set re {(href="/q\?s=)([^&]*)(&d=t")(.*)}
    set found [regexp -nocase $re $html match s1 sym s3 rest]
    while {$found} {
	# trim out security name
	set co ""
	regexp {(<small>)([^&]*)(&nbsp*.)} $rest match s1 co s3
	lappend symco_list $sym $co
	set html $rest
	set found [regexp -nocase $re $html match s1 sym s3 rest]
    }
    return $symco_list
}


